import time
import requests
from pydantic import BaseModel, Field
from typing import Any, Dict, Union, Callable, Optional
from zlai.utils.mixin import LoggerMixin
from zlai.tools.utils import headers
from zlai.types.tools import ResponseData
from .base import *


__all__ = [
    "ProfitForecast"
]


class ProfitForecastConfig(BaseModel):
    """"""
    size: Optional[int] = Field(default=5, description="数据量")
    code: Optional[Union[str, int]] = Field(default=None, description="股票代码")
    industry: Optional[str] = Field(default=None, description="行业板块")
    district: Optional[str] = Field(default=None, description="地域板块")
    conception: Optional[str] = Field(default=None, description="概念板块")

    def __init__(self,**kwargs):
        super().__init__(**kwargs)

    def _query_condition(self) -> str:
        """"""
        if self.code:
            condition = f'(SECURITY_CODE="{self.code}")'
        elif self.industry:
            condition = f'(INDUSTRY_BOARD="{self.industry}")'
        elif self.district:
            condition = f'(REGION_BOARD="{self.district}")'
        elif self.conception:
            condition = f'(CONCEPTINDEX_BOARD+like"%{self.conception}%")'
        else:
            condition = ""
        return condition

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": "RPT_WEB_RESPREDICT",
            "columns": "WEB_RESPREDICT",
            "pageNumber": "1",
            "pageSize": self.size,
            "sortTypes": "-1",
            "sortColumns": "RATING_ORG_NUM",
            "filter": self._query_condition(),
            "p": "1",
            "pageNo": "1",
            "pageNum": "1",
            "_": str(int(time.time() * 1E3)),
        }
        return params


class ProfitForecast(LoggerMixin):
    """"""
    def __init__(
            self,
            query_config: Optional[ProfitForecastConfig] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """"""
        if query_config is None:
            self.query_config = ProfitForecastConfig.model_validate(kwargs)
        else:
            self.query_config = query_config
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

    def load_data(self, ) -> ResponseData:
        """"""
        base_url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
        params = self.query_config.to_params()
        response = requests.get(base_url, params=params, headers=headers)
        metadata = response.json().get("result", {})
        data = metadata.pop("data")
        self.update_metadata(metadata)
        self._logger(msg=f"[{__class__.__name__}] Find {len(data)} reports.", color="green")
        return ResponseData(data=data, metadata=metadata)

    def update_metadata(self, metadata: Dict):
        """"""
        metadata.update({
            "description": "机构盈利预测",
            "columns": {
                "SECUCODE": "股票代码",
                "SECURITY_CODE": "证券代码",
                "SECURITY_NAME_ABBR": "证券简称",
                "RATING_ORG_NUM": "评级机构数量",
                "RATING_BUY_NUM": "买入评级数量",
                "RATING_ADD_NUM": "增持评级数量",
                "RATING_NEUTRAL_NUM": "中性评级数量",
                "RATING_REDUCE_NUM": "减持评级数量",
                "RATING_SALE_NUM": "卖出评级数量",
                "YEAR1": "第一年",
                "YEAR_MARK1": "第一年标记",
                "EPS1": "第一年每股收益",
                "YEAR2": "第二年",
                "YEAR_MARK2": "第二年标记",
                "EPS2": "第二年每股收益",
                "YEAR3": "第三年",
                "YEAR_MARK3": "第三年标记",
                "EPS3": "第三年每股收益",
                "YEAR4": "第四年",
                "YEAR_MARK4": "第四年标记",
                "EPS4": "第四年每股收益",
                "INDUSTRY_BOARD": "行业板块",
                "INDUSTRY_BOARD_SZM": "深证行业板块",
                "CONCEPTINDEX_BOARD": "概念指数板块",
                "CONCEPTINDEX_BOARD_SZM": "深证概念指数板块",
                "REGION_BOARD": "区域板块",
                "REGION_BOARD_SZM": "深证区域板块",
                "MARKET_BOARD": "市场板块",
                "DEC_AIMPRICEMAX": "目标最高价",
                "DEC_AIMPRICEMIN": "目标最低价",
                "RATING_LONG_NUM": "长期评级数量",
            }
        })



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "FDIMapping",
    "FDI",
]


class FDIMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "ACTUAL_FOREIGN": "当月(亿美元)",
        "ACTUAL_FOREIGN_SAME": "同比增长",
        "ACTUAL_FOREIGN_SEQUENTIAL": "环比增长",
        "ACTUAL_FOREIGN_ACCUMULATE": "累计(亿美元)",
        "FOREIGN_ACCUMULATE_SAME": "同比增长",
    }


class FDI(MacroRequestData):
    """中国 外商直接投资数据(FDI)"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = FDIMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 外商直接投资数据(FDI)")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "ACTUAL_FOREIGN", "ACTUAL_FOREIGN_SAME",
            "ACTUAL_FOREIGN_SEQUENTIAL", "ACTUAL_FOREIGN_ACCUMULATE", "FOREIGN_ACCUMULATE_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_FDI",
        }
        return self.base_param(update=params)



import time
import requests
from typing import Dict
from zlai.utils.mixin import *
from zlai.tools.utils import headers
from zlai.types.tools import ResponseData


__all__ = [
    "ReportMapping"
]


class ReportMapping(LoggerMixin):
    """"""
    base_url: str = "https://reportapi.eastmoney.com/report/bk"
    mapping_code: Dict[str, str] = {
        "地域板块": "020",
        "行业板块": "016",
        "概念板块": "007",
    }

    def _get_data(self, params: Dict) -> ResponseData:
        """"""
        response = requests.get(self.base_url, params=params, headers=headers)
        metadata = response.json()
        data = metadata.pop("data")
        return ResponseData(data=data, metadata=metadata)

    def list_industry(self, ) -> ResponseData:
        """"""
        params = {"bkCode": "016", "_": str(int(time.time() * 1E3))}
        return self._get_data(params)

    def list_conception(self) -> ResponseData:
        """"""
        params = {"bkCode": "007", "_": str(int(time.time() * 1E3))}
        return self._get_data(params)

    def list_district(self) -> ResponseData:
        """"""
        params = {"bkCode": "020", "_": str(int(time.time() * 1E3))}
        return self._get_data(params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "GovIncomeMapping",
    "GovIncome",
]


class GovIncomeMapping(BaseMapping):
    """"""
    columns: Dict = {
        "START_DATE": "开始时间",
        "END_DATE": "截至时间",
        "BASE": "当月(亿元)",
        "BASE_SAME": "同比增长",
        "BASE_SEQUENTIAL": "环比增长",
        "BASE_ACCUMULATE": "累计(亿元)",
        "ACCUMULATE_SAME": "累计同比增长",
    }


class GovIncome(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = GovIncomeMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 财政收入（每年2月公布当年1-2月累计值）")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "BASE", "BASE_SAME", "BASE_SEQUENTIAL", "BASE_ACCUMULATE", "ACCUMULATE_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_INCOME",
        }
        return params



import os
import time
import random
import requests
from pydantic import BaseModel, Field
from typing import Any, List, Dict, Tuple, Union, Literal, Callable, Optional
from zlai.tools.utils import headers
from zlai.utils.mixin import LoggerMixin
from zlai.types.tools import ResponseData


__all__ = [
    "ReportQueryConfig",
    "Report",
]


TypeReport = Literal["个股研报", "行业研报", "策略报告", "宏观研究", "券商晨报"]


class ReportQueryConfig(BaseModel):
    """"""
    code: Optional[str] = Field(default=None)
    industry: Optional[str] = Field(default="*")
    industry_code: Optional[Union[str, int]] = Field(default="*")
    size: Optional[int] = Field(default=5)
    rating: Optional[str] = Field(default="*")
    rating_change: Optional[str] = Field(default="*")
    begin_time: Optional[str] = Field(default="*")
    end_time: Optional[str] = Field(default="*")
    page_no: Optional[int] = Field(default=1)
    report_type: Optional[TypeReport] = Field(default="*")
    q_type: Optional[Union[int, str]] = Field(default=None)

    def __init__(self, **kwargs):
        """
        :param code:
        :param industry:
        :param industry_code:
        :param size:
        :param rating:
        :param rating_change:
        :param begin_time:
        :param end_time:
        :param page_no:
        :param report_type: 个股研报 行业研报 策略报告 宏观研究 券商晨报
        """
        super().__init__(**kwargs)

    def _reports_type(self):
        """"""
        q_types = ["个股研报", "行业研报", "策略报告", "宏观研究", "券商晨报"]
        if self.report_type is None or self.report_type == "*":
            self.q_type = "*"
        elif self.report_type in q_types:
            self.q_type = q_types.index(self.report_type)

    def _random_cb(self) -> str:
        """"""
        return str(int(random.random() * 1E7 + 1))

    def _current_time(self) -> str:
        """"""
        return str(int(time.time() * 1E3))

    def params(self) -> Dict:
        """"""
        if self.q_type is None:
            self._reports_type()

        params = {
            "industry": self.industry,
            "industryCode": self.industry_code,
            "pageSize": self.size,
            "rating": self.rating,
            "ratingChange": self.rating_change,
            "beginTime": self.begin_time,
            "endTime": self.end_time,
            "pageNo": self.page_no,
            "qType": self.q_type,
            "_": self._current_time()
        }
        if self.code:
            params.update({"code": self.code})
        return params


class Report(LoggerMixin):
    """"""
    def __init__(
            self,
            query_config: Optional[ReportQueryConfig] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        if query_config is None:
            self.query_config = ReportQueryConfig.model_validate(kwargs)
        else:
            self.query_config = query_config
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

    def _base_url(self) -> str:
        """jg/dg/list"""
        if self.query_config.q_type is None:
            self.query_config._reports_type()

        base_url = "https://reportapi.eastmoney.com/report/"
        if self.query_config.q_type in [0, 1,]:
            base_url += "list"
        else:
            base_url += "dg"
        return base_url

    def load_data(
            self,
    ) -> ResponseData:
        """"""
        base_url = self._base_url()
        params = self.query_config.to_params()
        response = requests.get(base_url, params=params, headers=headers)
        data = response.json()
        reports = data.pop("data")
        metadata = data
        self._logger(msg=f"[{__class__.__name__}] Find {len(reports)} reports.", color="green")
        return ResponseData(data=reports, metadata=metadata)

    def save_pdf(
            self,
            report_code: Union[str, List[str]],
            path: str = "./"
    ) -> None:
        """"""
        pdf_bytes = self.load_pdf_bytes(report_code=report_code)
        for code, bytes_content in pdf_bytes:
            if isinstance(bytes_content, bytes):
                file_name = f"{code}.pdf"
                file_path = os.path.join(path, file_name)
                with open(file_path, "wb") as f:
                    f.write(bytes_content)

    def load_pdf_bytes(self, report_code: Union[str, List[str]], ) -> List[Tuple[str, Union[str, bytes]]]:
        """"""
        base_url = """https://pdf.dfcfw.com/pdf/H3_{}_1.pdf""".format
        pdf_bytes = []

        if isinstance(report_code, str):
            report_code = [report_code]

        for code in report_code:
            try:
                response = requests.get(base_url(code), headers=headers)
                pdf_bytes.append((code, response.content))
            except Exception as e:
                pdf_bytes.append((code, str(e)))
                self._logger(msg=f"[{__class__.__name__}] Load `{code}` error, error message: {e}", color="red")
        return pdf_bytes



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "HolderChange",
    "HolderChangeMapping",
]


class HolderChangeMapping(BaseMapping):
    """字段映射 上市公司十大股东持股变动统计"""
    columns: Dict = {
        "HOLDER_NEW": "新的股东",
        "END_DATE": "截止日期",
        "HOLDER_NAME": "股东姓名",
        "HOLDER_CODE": "股东代码",
        "HOLDER_TYPE": "股东类型",
        "REPORT_DATE_NAME": "报告名称",
        "HOLDER_SOURCE_CODE": "股东来源代码",
        "HOLDER_SOURCE": "股东来源",
        "HOLDER_NUM": "持有股数",
        "HOLDADD_NUM": "增持股数",
        "HOLDUP_NUM": "持股增加数",
        "HOLDDOWN_NUM": "持股减少数",
        "HOLDUNCHANGED_NUM": "持股不变数",
        "IS_REPORT": "是否报告",
        "HOLDER_MARKET_CAP": "股东市值",
        "SEAB_JOIN": "关联股票",
        "CLOSE_PRICE": "收盘价",
        "IS_MAX_REPORTDATE": "是否为最新报告日期"
    }


class HolderChange(APIDataV1RequestData):
    """查询 上市公司十大股东持股变动统计"""
    def __init__(
            self,
            holder_name: Optional[str] = None,
            holder_type: Optional[Literal["个人", "基金", "QFII", "社保", "券商", "信托"]] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.holder import *

            data = HolderChange(size=20, start_date="2024-09-30").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            holder_name: 股东名称
            holder_type: 股东类型 `["个人", "基金", "QFII", "社保", "券商", "信托"]`
            size: 返回条数
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否显示日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.holder_name = holder_name
        self.holder_type = holder_type
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = HolderChangeMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司十大股东持股变动统计")
        self.conditions = []

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="END_DATE")
        if self.holder_type:
            self.conditions.append(f'(HOLDER_TYPE="{self.holder_type}")')
        if self.holder_name:
            self.conditions.append(f'(HOLDER_NAME+like+"%{self.holder_name}%")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "HOLDER_NUM,HOLDER_NEW",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_HOLDERS_BASIC_INFONEW",
        }
        return self.base_param(params)
